function [w, ep] = minlipl1(model,Y,delta,gam)
% Implementation of minlipl1 using Yalmip
% Use censored failure time data
% event(not censored): delta=1;  censored: delta=0
%
% >> w=minlipl1(model)
% >> w=minlipl1(X,Y,delta,gam)
% >> [w,ep]=minlipl1(model)
% >> [w,ep]=minlipl1(X,Y,delta,gam)
%
% Input:
% Case 1: Structure format:
% 'model' is a structure which includes the fields 'func','X','Y','delta','gam','mu'
% Case 2: Original format:
% 'X' input: is a matrix of n x d
% 'Y' time: is a vector of n elements
% 'delta' censoring: is a vector of n elements (0-1 values, 0 means events and 1 means censored)
% 'gam' a parameter: is a scalar
%
% Output:
% 'w' parameter: is a vector of d elements
% 'ep' residuals: is a vector of n elements
%
% Author: Liu,Yang (liu.yang@it.uu.se)

% Selection of input format
if nargin==1
    [~,X,Y,delta,gam]=ungroup(model);
else
    X=model;
end

[n,d]=size(X);

% Sort by Y
[Y,events] = sort(Y);
X= X(events,:);
delta=delta(events,:);

% Build matrix D and Dc
Dc=buildDMatrix(n,delta,Y);

% Solve QP by solver
yalmip('clear');
w=sdpvar(d,1);
ep=sdpvar(n,1);
solvesdp(Dc*(X*w+ep)>=Dc*Y,...
    norm(w,1)+gam*norm(ep,1),...
    sdpsettings('solver','mosek','cachesolvers',1,'verbose',0));

% Convert the result to double type
w=double(w);
ep=double(ep);